import { useMarkStore } from "@/store";
import { saveTask, doUnselectiveType } from "@/api/markPage";
import { message } from "ant-design-vue";
import { isNumber } from "lodash-es";
import { h } from "vue";
import EventBus from "@/plugins/eventBus";
import type { Question } from "@/types";
import useStatus from "./useStatus";
import { useMarkTask } from "./useMarkTask";

export function useMarkSubmit() {
  const markStore = useMarkStore();
  const { updateStatus } = useStatus();
  const { nextTask } = useMarkTask();

  const validateScore = (markResult: any) => {
    const errors: Array<{ question: Question; index: number; error: string }> =
      [];
    markResult.scoreList.forEach((score: number, index: number) => {
      if (!markStore.currentTask) return;
      const question = markStore.currentTask.questionList[index]!;
      let error;
      if (!isNumber(score)) {
        error = `${question.mainNumber}-${question.subNumber}${
          question.questionName ? "（" + question.questionName + "）" : ""
        } 没有给分，不能提交。`;
      } else if (isNumber(question.maxScore) && score > question.maxScore) {
        error = `${question.mainNumber}-${question.subNumber}${
          question.questionName ? "（" + question.questionName + "）" : ""
        } 给分大于最高分不能提交。`;
      } else if (isNumber(question.minScore) && score < question.minScore) {
        error = `${question.mainNumber}-${question.subNumber}${
          question.questionName ? "（" + question.questionName + "）" : ""
        } 给分小于最低分不能提交。`;
      }
      if (error) {
        errors.push({ question, index, error });
      }
    });
    return errors;
  };

  const saveTaskToServer = async () => {
    if (!markStore.currentTask) return;
    const markResult = markStore.currentTask.markResult;
    if (!markResult) return;

    const mkey = "save_task_key";
    const errors = validateScore(markResult);

    if (errors.length !== 0) {
      console.log(errors);
      const msg = errors.map((v) => h("div", `${v.error}`));
      void message.warning({
        content: h("span", ["校验失败", ...msg]),
        duration: 10,
        key: mkey,
      });
      return;
    }

    if (
      markResult.scoreList.length !==
        markStore.currentTask.questionList.length ||
      !markResult.scoreList.every((s) => isNumber(s))
    ) {
      console.error({ content: "markResult格式不正确，缺少分数", key: mkey });
      return;
    }

    if (!markStore.isTrackMode) {
      markResult.trackList = [];
    } else {
      const trackScores =
        markResult.trackList
          .map((t) => Math.round((t.score || 0) * 100))
          .reduce((acc, s) => acc + s, 0) / 100;
      if (trackScores !== markResult.markerScore) {
        void message.error({
          content: "轨迹分与总分不一致，请检查。",
          duration: 3,
          key: mkey,
        });
        return;
      }
    }

    if (markStore.setting.forceSpecialTag) {
      if (
        markResult.trackList.length === 0 &&
        markResult.specialTagList.length === 0
      ) {
        void message.error({
          content: "强制标记已开启，请至少使用一个标记。",
          duration: 5,
          key: mkey,
        });
        return;
      }
    }

    console.log("save task to server");
    void message.loading({ content: "保存评卷任务...", key: mkey });
    const res = await saveTask();
    if (!res) return;

    // 故意不在此处同步等待，因为不必等待
    updateStatus().catch((e) => console.log("保存任务后获取status出错", e));
    if (res.data.success && markStore.currentTask) {
      void message.success({ content: "保存成功", key: mkey, duration: 2 });
      if (!markStore.historyOpen) {
        markStore.currentTask = undefined;
        markStore.tasks.shift();
        markStore.currentTask = markStore.tasks[0];
      } else {
        EventBus.emit("should-reload-history");
      }
    } else if (!res.data.success) {
      void message.error({
        content: "提交失败，请刷新页面",
        key: mkey,
        duration: 10,
      });
      return;
    } else if (!markStore.currentTask) {
      void message.warn({ content: "暂无新任务", key: mkey, duration: 10 });
    }

    // 获取下一个任务
    void nextTask();
  };

  const allZeroSubmit = async () => {
    const markResult = markStore.currentTask?.markResult;
    if (!markResult) return;

    const { markerScore, scoreList, trackList, specialTagList } = markResult;
    markResult.markerScore = 0;
    const ss = new Array(markStore.currentTaskEnsured.questionList.length);
    markResult.scoreList = ss.fill(0);
    markResult.trackList = [];

    try {
      await saveTaskToServer();
    } catch (error) {
      console.log("error restore");
    } finally {
      markResult.markerScore = markerScore;
      markResult.scoreList = scoreList;
      markResult.trackList = trackList;
      markResult.specialTagList = specialTagList;
    }
  };

  const unselectiveSubmit = async () => {
    const markResult = markStore.currentTask?.markResult;
    if (!markResult) return;

    try {
      const res = await doUnselectiveType();
      if (res?.data.success) {
        void message.success({ content: "未选做处理成功", duration: 3 });
        if (!markStore.historyOpen) {
          markStore.currentTask = undefined;
          markStore.tasks.shift();
          markStore.currentTask = markStore.tasks[0];
        }
        if (markStore.historyOpen) {
          EventBus.emit("should-reload-history");
        }
        await updateStatus();
      } else {
        void message.error({
          content: res?.data.message || "错误",
          duration: 5,
        });
      }
    } catch (error) {
      console.log("未选做处理失败", error);
      void message.error({ content: "网络异常", duration: 5 });
      await new Promise((res) => setTimeout(res, 1500));
      window.location.reload();
    }
  };

  return {
    saveTaskToServer,
    allZeroSubmit,
    unselectiveSubmit,
  };
}
